@@ -22,11 +22,12 @@
 # code, but you are not obligated to do so.  If you do not wish to do so,
 # delete this exception statement from your version.
 
-from gi.repository import Gtk, Gdk, Gio, GLib, GObject, Handy
+from typing import Optional
+
+from gi.repository import Adw, Gtk, Gdk, Gio, GLib, GObject
 from gettext import gettext as _
 
 from gnomemusic.gstplayer import Playback
-from gnomemusic.mediakeys import MediaKeys
 from gnomemusic.player import RepeatMode
 from gnomemusic.search import Search
 from gnomemusic.trackerwrapper import TrackerState
@@ -38,7 +39,6 @@ from gnomemusic.views.searchview import SearchView
 from gnomemusic.views.songsview import SongsView
 from gnomemusic.views.playlistsview import PlaylistsView
 from gnomemusic.widgets.headerbar import HeaderBar
-from gnomemusic.widgets.notificationspopup import NotificationsPopup  # noqa
 from gnomemusic.widgets.playertoolbar import PlayerToolbar  # noqa: F401
 from gnomemusic.widgets.playlistdialog import PlaylistDialog
 from gnomemusic.widgets.searchheaderbar import SearchHeaderBar
@@ -47,7 +47,7 @@ from gnomemusic.windowplacement import WindowPlacement
 
 
 @Gtk.Template(resource_path="/org/gnome/Music/ui/Window.ui")
-class Window(Handy.ApplicationWindow):
+class Window(Adw.ApplicationWindow):
 
     __gtype_name__ = "Window"
 
@@ -55,13 +55,13 @@ class Window(Handy.ApplicationWindow):
     selected_songs_count = GObject.Property(type=int, default=0, minimum=0)
     selection_mode = GObject.Property(type=bool, default=False)
 
-    notifications_popup = Gtk.Template.Child()
     _headerbar_stack = Gtk.Template.Child()
-    _key_controller = Gtk.Template.Child()
+    _loading_progress = Gtk.Template.Child()
     _overlay = Gtk.Template.Child()
     _player_toolbar = Gtk.Template.Child()
     _selection_toolbar = Gtk.Template.Child()
     _stack = Gtk.Template.Child()
+    _toast_overlay = Gtk.Template.Child()
 
     def __init__(self, app):
         """Initialize the main window.
@@ -90,14 +90,14 @@ class Window(Handy.ApplicationWindow):
         self._current_view = None
         self._view_before_search = None
 
+        self._playlist_dialog: Optional[PlaylistDialog] = None
+
         self._player = app.props.player
         self._search = app.props.search
 
         self._startup_timeout_id = 0
         self._setup_view()
 
-        MediaKeys(self._player, self)
-
     def _setup_view(self):
         self._headerbar = HeaderBar(self._app)
         self._headerbar.props.stack = self._stack
@@ -228,11 +228,11 @@ class Window(Handy.ApplicationWindow):
         if self.views[View.ALBUM] is not None:
             return
 
-        self._btn_ctrl = Gtk.GestureMultiPress().new(self)
-        self._btn_ctrl.props.propagation_phase = Gtk.PropagationPhase.CAPTURE
+        ctrl = Gtk.GestureClick().new()
         # Mouse button 8 is the back button.
-        self._btn_ctrl.props.button = 8
-        self._btn_ctrl.connect("pressed", self._on_back_button_pressed)
+        ctrl.props.button = 8
+        ctrl.connect("pressed", self._on_back_button_pressed)
+        self.add_controller(ctrl)
 
         self.views[View.EMPTY].props.state = EmptyView.State.SEARCH
 
@@ -249,9 +249,9 @@ class Window(Handy.ApplicationWindow):
         # from being displayed during startup
         for i in self.views[View.ALBUM:]:
             if i.props.title:
-                self._stack.add_titled(i, i.props.name, i.props.title)
-                self._stack.child_set_property(
-                    i, "icon-name", i.props.icon_name)
+                stackpage = self._stack.add_titled(
+                    i, i.props.name, i.props.title)
+                stackpage.props.icon_name = i.props.icon_name
             else:
                 self._stack.add_named(i, i.props.name)
 
@@ -287,7 +287,7 @@ class Window(Handy.ApplicationWindow):
         modifiers = state & Gtk.accelerator_get_default_mod_mask()
         control_mask = Gdk.ModifierType.CONTROL_MASK
         shift_mask = Gdk.ModifierType.SHIFT_MASK
-        mod1_mask = Gdk.ModifierType.MOD1_MASK
+        alt_mask = Gdk.ModifierType.ALT_MASK
         shift_ctrl_mask = control_mask | shift_mask
 
         # Ctrl+<KEY>
@@ -327,7 +327,7 @@ class Window(Handy.ApplicationWindow):
             if keyval == Gdk.KEY_A:
                 self._deselect_all()
         # Alt+<KEY>
-        elif modifiers == mod1_mask:
+        elif modifiers == alt_mask:
             # Go back from child view on Alt + Left
             if keyval == Gdk.KEY_Left:
                 self._switch_back_from_childview()
@@ -442,9 +442,22 @@ class Window(Handy.ApplicationWindow):
     def _on_selection_mode_changed(self, widget, data=None):
         if (not self.props.selection_mode
                 and self._player.state == Playback.STOPPED):
-            self._player_toolbar.hide()
+            self._player_toolbar.props.revealed = False
+
+    def _on_add_to_playlist(self, widget: SelectionToolbar) -> None:
+
+        def on_response(dialog: PlaylistDialog, response_id: int) -> None:
+            if not self._playlist_dialog:
+                return
+
+            if response_id == Gtk.ResponseType.ACCEPT:
+                playlist = self._playlist_dialog.props.selected_playlist
+                playlist.add_songs(selected_songs)
+
+            self.props.selection_mode = False
+            self._playlist_dialog.destroy()
+            self._playlist_dialog = None
 
-    def _on_add_to_playlist(self, widget):
         if self.props.active_view == self.views[View.PLAYLIST]:
             return
 
@@ -453,18 +466,31 @@ class Window(Handy.ApplicationWindow):
         if len(selected_songs) < 1:
             return
 
-        playlist_dialog = PlaylistDialog(self._app)
-        playlist_dialog.props.transient_for = self
-        if playlist_dialog.run() == Gtk.ResponseType.ACCEPT:
-            playlist = playlist_dialog.props.selected_playlist
-            playlist.add_songs(selected_songs)
-
-        self.props.selection_mode = False
-        playlist_dialog.destroy()
+        self._playlist_dialog = PlaylistDialog(self._app)
+        self._playlist_dialog.props.transient_for = self
+        self._playlist_dialog.connect("response", on_response)
+        self._playlist_dialog.present()
 
     def set_player_visible(self, visible):
         """Set PlayWidget action visibility
 
         :param bool visible: actionbar visibility
         """
-        self._player_toolbar.set_visible(visible)
+        self._player_toolbar.props.revealed = visible
+
+    def loading_pulse(self) -> bool:
+        """Pulse the loading progress bar
+
+        :returns: GLib.SOURCE_CONTINUE
+        :rtype: bool
+        """
+        self._loading_progress.pulse()
+
+        return GLib.SOURCE_CONTINUE
+
+    def loading_visible(self, show: bool) -> None:
+        """Sets visibility of the loading progressbar
+
+        :param bool show: Wheter to show the loading bar
+        """
+        self._loading_progress.props.visible = show
